Model subclass: #Node
	instanceVariableNames: 'name nextNode '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lan-Simulation'!
Node comment:
'I represent a basic node in the LAN framework, with a name and a next node. I can receive packets with the accept: method, and send them to my nextnode with the send: method.



Instance Variables:

	name: my name

	nextNode: the next node I point to.'!


!Node methodsFor: 'initialize-release'!

initialize
	"I am the default initialize method. My subclasses might want to override me to do something usefull"! !

!Node methodsFor: 'sending/receiving'!

accept: thePacket
	"Having received the packet, send it on. This is the default behavior"
	"My subclasses will probably override me to do something special"

	self send: thePacket!

send: thePacket
	"Display debug information in the Transcript, then send the packet to the node with whom I communicate"

	Transcript show: 
		self name printString,' 
		sending the packet to ',
		self nextNode name printString; cr.
	self nextNode accept: thePacket! !

!Node methodsFor: 'printing'!

printOn: aStream
	"I add a textual representation of myself on my argument, aStream"

	super printOn: aStream.
	aStream nextPutAll: ' with name: ' ; print: self name.
	self hasNextNode  ifFalse: [
		aStream nextPutAll: ' and next node: '; print: self nextNode name]! !

!Node methodsFor: 'private'!

name
	^name!

name: aSymbol
	name := aSymbol!

nextNode
	^nextNode!

nextNode: aNode
	nextNode := aNode! !

!Node methodsFor: 'accessing'!

hasNextNode

	^ self nextNode = self class noNextNode! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Node class
	instanceVariableNames: ''!


!Node class methodsFor: 'constant'!

noNextNode
	^ nil! !

!Node class methodsFor: 'instance creation'!

new
	self error: 'Node should be created using withName: or withName:nextNode:'!

withName: aSymbol
	^ self  withName: aSymbol nextNode: self noNextNode!

withName: aSymbol nextNode: aNode

	^ self basicNew initialize name: aSymbol ; nextNode: aNode! !

Object subclass: #LAN
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lan-Simulation'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LAN class
	instanceVariableNames: ''!


!LAN class methodsFor: 'settings'!

settings1 
	"self settings1"

	|mac pc node1 node2 node3 dinfNT2 igPrinter packet|
	mac := Workstation withName: #mac.
	pc := Workstation withName: #pc.
	node1 := Node withName: #node1.
	node2 := Node withName: #node2.
	node3 := Node withName: #node3.
	dinfNT2 := FileServer withName: #DinfNT2.
	igPrinter := PrintServer withName: #IGPrinter.

	"connect the different nodes." 
	"I  make following connections:mac -> node1 -> dinfNT2 -> node2 -> igPrinter -> node3 -> pc -> mac"
	mac nextNode: node1.
	node1 nextNode: dinfNT2.
	dinfNT2 nextNode: node2.
	node2 nextNode: igPrinter.
	igPrinter nextNode: node3.
	node3 nextNode: pc.
	pc nextNode: mac.

	"create a packet and start simulation (try this one first)"
	packet := Packet send:  'This packet travelled around to the printer' to:  #IGPrinter.
	mac originate: packet.

	"another one (try this afterwards)"
	packet := Packet send: 'This packet travels around to the printer' to: #anotherPrinter.
	pc originate: packet! !

Node subclass: #FileServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lan-Simulation'!


!FileServer methodsFor: 'sending/receiving'!

accept: thePacket
	"If the packet is addressed to me, I save it. Otherwise I let my superclass handle it"


	(self name = thePacket addressee)
		ifTrue: [self save: thePacket]
		ifFalse: [super accept: thePacket]! !

!FileServer methodsFor: 'saving'!

save: thePacket
	"I have to save the packet."

	Transcript show: 
		'fileserver ', 
		self name printString ,
		' saving the packet with contents: ',
		thePacket contents printString! !

Object subclass: #Packet
	instanceVariableNames: 'contents addressee originator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lan-Simulation'!


!Packet methodsFor: 'printing'!

printOn: aStream
	"I add a textual representation of myself on my argument, aStream"

	super printOn: aStream.
	aStream nextPutAll: ' addressed to: '; print: self addressee.
	aStream nextPutAll: ' with contents:'; print: self contents! !

!Packet methodsFor: 'accessing'!

isAddressedTo: aNode
	
	^ self addressee = aNode name!

isOriginatingFrom: aNode

	^ self originator = aNode! !

!Packet methodsFor: 'initialize-release'!

initialize
	"root"
	contents := 'Default contents'! !

!Packet methodsFor: 'private'!

addressee

	^addressee!

addressee: aSymbol

	addressee := aSymbol!

contents

	^contents!

contents: aString

	contents := aString!

originator

	^originator!

originator: aNode

	originator := aNode! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Packet class
	instanceVariableNames: ''!


!Packet class methodsFor: 'instance creation'!

new
	self error: 'Packet should only be created using send:to:'!

send: aString to: anAddress 

	^self basicNew initialize contents: aString ; addressee: anAddress ; yourself! !

Node subclass: #PrintServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lan-Simulation'!


!PrintServer methodsFor: 'printing'!

print: thePacket

	"I have to print the packet. Write this on the transcript"



	Transcript show: 'printer ',self name printString,' printing the packet with contents: ',thePacket contents printString! !

!PrintServer methodsFor: 'sending/receiving'!

accept: thePacket
	"If the packet is addressed to me, I print it. Otherwise I let my superclass handle it"

	( thePacket isAddressedTo: self)
		ifTrue: [self print: thePacket]
		ifFalse: [super accept: thePacket]! !

Node subclass: #Workstation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lan-Simulation'!


!Workstation methodsFor: 'sending/receiving'!

accept: thePacket

	(thePacket isOriginatingFrom: self)
		ifTrue: [Transcript show: 'Packet taken out because of invalid addressee (',thePacket addressee printString,')']
		ifFalse: [super accept: thePacket]! !

!Workstation methodsFor: 'originating'!

originate: thePacket

	"This is how packets get inserted into the network. 
	This is a likely method to be rewritten to permit packets to be entered in various ways. 
	Currently, I assume that someone else creates the packet and passes it to me as an argument."

	thePacket originator: self.
	self send: thePacket! !

